@@ -1,20 +1,21 @@
-/* Copyright (C) 2012, 2013 Free Software Foundation, Inc.
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 2012-2013,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -25,21 +26,32 @@
 
 #include <assert.h>
 
-#include "libguile/_scm.h"
-#include "libguile/control.h"
-#include "libguile/eval.h"
-#include "libguile/fluids.h"
-#include "libguile/dynstack.h"
+#ifndef _WIN64
+#include <setjmp.h>
+#else
+#include "setjump-win.h"
+#endif
+
+#include "control.h"
+#include "eval.h"
+#include "fluids.h"
+#include "variable.h"
+#include "threads.h"
+
+#include "dynstack.h"
 
 
 
 
-#define PROMPT_WORDS 5
+#define PROMPT_WORDS 6
 #define PROMPT_KEY(top) (SCM_PACK ((top)[0]))
-#define PROMPT_FP(top) ((scm_t_ptrdiff) ((top)[1]))
-#define PROMPT_SP(top) ((scm_t_ptrdiff) ((top)[2]))
-#define PROMPT_IP(top) ((scm_t_uint32 *) ((top)[3]))
-#define PROMPT_JMPBUF(top) ((scm_i_jmp_buf *) ((top)[4]))
+#define PROMPT_FP(top) ((ptrdiff_t) ((top)[1]))
+#define SET_PROMPT_FP(top, fp) do { top[1] = (scm_t_bits)(fp); } while (0)
+#define PROMPT_SP(top) ((ptrdiff_t) ((top)[2]))
+#define SET_PROMPT_SP(top, sp) do { top[2] = (scm_t_bits)(sp); } while (0)
+#define PROMPT_VRA(top) ((uint32_t *) ((top)[3]))
+#define PROMPT_MRA(top) ((uint8_t *) ((top)[4]))
+#define PROMPT_JMPBUF(top) ((jmp_buf *) ((top)[5]))
 
 #define WINDER_WORDS 2
 #define WINDER_PROC(top) ((scm_t_guard) ((top)[0]))
@@ -113,6 +125,7 @@ push_dynstack_entry_unchecked (scm_t_dynstack *dynstack,
 
   SCM_DYNSTACK_SET_TAG (dynstack->top, SCM_MAKE_DYNSTACK_TAG (type, flags, len));
   dynstack->top += SCM_DYNSTACK_HEADER_LEN + len;
+  SCM_DYNSTACK_SET_TAG (dynstack->top, 0);
   SCM_DYNSTACK_SET_PREV_OFFSET (dynstack->top, SCM_DYNSTACK_HEADER_LEN + len);
 
   return ret;
@@ -189,8 +202,8 @@ void
 scm_dynstack_push_prompt (scm_t_dynstack *dynstack,
                           scm_t_dynstack_prompt_flags flags,
                           SCM key,
-                          scm_t_ptrdiff fp_offset, scm_t_ptrdiff sp_offset,
-                          scm_t_uint32 *ip, scm_i_jmp_buf *registers)
+                          ptrdiff_t fp_offset, ptrdiff_t sp_offset,
+                          uint32_t *vra, uint8_t *mra, jmp_buf *registers)
 {
   scm_t_bits *words;
 
@@ -199,8 +212,9 @@ scm_dynstack_push_prompt (scm_t_dynstack *dynstack,
   words[0] = SCM_UNPACK (key);
   words[1] = (scm_t_bits) fp_offset;
   words[2] = (scm_t_bits) sp_offset;
-  words[3] = (scm_t_bits) ip;
-  words[4] = (scm_t_bits) registers;
+  words[3] = (scm_t_bits) vra;
+  words[4] = (scm_t_bits) mra;
+  words[5] = (scm_t_bits) registers;
 }
 
 void
@@ -286,6 +300,24 @@ scm_dynstack_capture (scm_t_dynstack *dynstack, scm_t_bits *item)
   return ret;
 }
 
+void
+scm_dynstack_relocate_prompts (scm_t_dynstack *dynstack, ptrdiff_t base)
+{
+  scm_t_bits *walk;
+
+  /* Relocate prompts.  */
+  for (walk = dynstack->top; walk; walk = SCM_DYNSTACK_PREV (walk))
+    {
+      scm_t_bits tag = SCM_DYNSTACK_TAG (walk);
+
+      if (SCM_DYNSTACK_TAG_TYPE (tag) == SCM_DYNSTACK_TYPE_PROMPT)
+        {
+          SET_PROMPT_FP (walk, PROMPT_FP (walk) - base);
+          SET_PROMPT_SP (walk, PROMPT_SP (walk) - base);
+        }
+    }
+}
+
 void
 scm_dynstack_wind_1 (scm_t_dynstack *dynstack, scm_t_bits *item)
 {
@@ -474,8 +506,8 @@ scm_dynstack_unwind_fork (scm_t_dynstack *dynstack, scm_t_dynstack *branch)
 scm_t_bits*
 scm_dynstack_find_prompt (scm_t_dynstack *dynstack, SCM key,
                           scm_t_dynstack_prompt_flags *flags,
-                          scm_t_ptrdiff *fp_offset, scm_t_ptrdiff *sp_offset,
-                          scm_t_uint32 **ip, scm_i_jmp_buf **registers)
+                          ptrdiff_t *fp_offset, ptrdiff_t *sp_offset,
+                          uint32_t **vra, uint8_t **mra, jmp_buf **registers)
 {
   scm_t_bits *walk;
 
@@ -493,8 +525,10 @@ scm_dynstack_find_prompt (scm_t_dynstack *dynstack, SCM key,
             *fp_offset = PROMPT_FP (walk);
           if (sp_offset)
             *sp_offset = PROMPT_SP (walk);
-          if (ip)
-            *ip = PROMPT_IP (walk);
+          if (vra)
+            *vra = PROMPT_VRA (walk);
+          if (mra)
+            *mra = PROMPT_MRA (walk);
           if (registers)
             *registers = PROMPT_JMPBUF (walk);
           return walk;
@@ -504,9 +538,59 @@ scm_dynstack_find_prompt (scm_t_dynstack *dynstack, SCM key,
   return NULL;
 }
 
+SCM
+scm_dynstack_find_old_fluid_value (scm_t_dynstack *dynstack, SCM fluid,
+                                   size_t depth, SCM dflt)
+{
+  scm_t_bits *walk;
+
+  for (walk = SCM_DYNSTACK_PREV (dynstack->top); walk;
+       walk = SCM_DYNSTACK_PREV (walk))
+    {
+      scm_t_bits tag = SCM_DYNSTACK_TAG (walk);
+
+      switch (SCM_DYNSTACK_TAG_TYPE (tag))
+        {
+        case SCM_DYNSTACK_TYPE_WITH_FLUID:
+          {
+            if (scm_is_eq (WITH_FLUID_FLUID (walk), fluid))
+              {
+                if (depth == 0)
+                  return SCM_VARIABLE_REF (WITH_FLUID_VALUE_BOX (walk));
+                else
+                  depth--;
+              }
+            break;
+          }
+        case SCM_DYNSTACK_TYPE_DYNAMIC_STATE:
+          {
+            SCM state, val;
+
+            /* The previous dynamic state may or may not have
+               established a binding for this fluid.  */
+            state = scm_variable_ref (DYNAMIC_STATE_STATE_BOX (walk));
+            val = scm_dynamic_state_ref (state, fluid, SCM_UNDEFINED);
+            if (!SCM_UNBNDP (val))
+              {
+                if (depth == 0)
+                  return val;
+                else
+                  depth--;
+              }
+            break;
+          }
+        default:
+          break;
+        }
+    }
+
+  return dflt;
+}
+
 void
 scm_dynstack_wind_prompt (scm_t_dynstack *dynstack, scm_t_bits *item,
-                          scm_t_ptrdiff reloc, scm_i_jmp_buf *registers)
+                          ptrdiff_t base_fp_offset,
+                          jmp_buf *registers)
 {
   scm_t_bits tag = SCM_DYNSTACK_TAG (item);
 
@@ -516,9 +600,10 @@ scm_dynstack_wind_prompt (scm_t_dynstack *dynstack, scm_t_bits *item,
   scm_dynstack_push_prompt (dynstack,
                             SCM_DYNSTACK_TAG_FLAGS (tag),
                             PROMPT_KEY (item),
-                            PROMPT_FP (item) - reloc,
-                            PROMPT_SP (item) - reloc,
-                            PROMPT_IP (item),
+                            PROMPT_FP (item) + base_fp_offset,
+                            PROMPT_SP (item) + base_fp_offset,
+                            PROMPT_VRA (item),
+                            PROMPT_MRA (item),
                             registers);
 }
 
@@ -593,9 +678,3 @@ scm_dynstack_unwind_dynamic_state (scm_t_dynstack *dynstack,
   clear_scm_t_bits (words, len);
 }
 
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
