@@ -1,4 +1,4 @@
-/* Copyright 1996-1998,2000-2007,2009,2011-2015,2018,2021,2022
+/* Copyright 1996-1998,2000-2007,2009,2011-2015,2018,2021,2022,2025
      Free Software Foundation, Inc.
 
    This file is part of Guile.
@@ -30,11 +30,11 @@
 #ifdef HAVE_STDINT_H
 #include <stdint.h>
 #endif
-#ifdef HAVE_STRING_H
+
 #include <string.h>
-#endif
 #include <unistd.h>
 #include <sys/types.h>
+#include <sys/time.h>
 
 #ifdef HAVE_WINSOCK2_H
 #include <winsock2.h>
@@ -406,7 +406,7 @@ SCM_DEFINE (scm_socket, "socket", 3, 0, 0,
 	    "@var{proto} can be obtained from a protocol name using\n"
 	    "@code{getprotobyname}.  A value of zero specifies the default\n"
 	    "protocol, which is usually right.\n\n"
-	    "A single socket port cannot by used for communication until it\n"
+	    "A single socket port cannot be used for communication until it\n"
 	    "has been connected to another socket.")
 #define FUNC_NAME s_scm_socket
 {
@@ -454,6 +454,7 @@ typedef union
 #endif
   size_t size;
   int    integer;
+  struct timeval timeval;
 } scm_t_getsockopt_result;
 
 SCM_DEFINE (scm_getsockopt, "getsockopt", 3, 0, 0,
@@ -502,6 +503,12 @@ SCM_DEFINE (scm_getsockopt, "getsockopt", 3, 0, 0,
 	    "timeout support (ie.@: without @code{struct linger}), only\n"
 	    "@var{enable} has an effect but the value in Guile is always a\n"
 	    "pair.\n"
+	    "@end defvar"
+            "\n"
+	    "@defvar SO_RCVTIMEO\n"
+	    "@defvarx SO_SNDTIMEO\n"
+	    "@var{value} is a pair of integers @code{(@var{SECONDS}\n"
+	    ". @var{MICROSECONDS})}.\n"
 	    "@end defvar")
 #define FUNC_NAME s_scm_getsockopt
 {
@@ -522,6 +529,16 @@ SCM_DEFINE (scm_getsockopt, "getsockopt", 3, 0, 0,
   if (getsockopt (fd, ilevel, ioptname, (void *) &optval, &optlen) == -1)
     SCM_SYSERROR;
 
+#if defined(SO_RCVTIMEO) && defined(SO_SNDTIMEO)
+  if (ioptname == SO_RCVTIMEO || ioptname == SO_SNDTIMEO)
+    {
+      struct timeval *opt_time = (struct timeval *) &optval;
+
+      return scm_cons (scm_from_long (opt_time->tv_sec),
+                       scm_from_long (opt_time->tv_usec));
+    }
+#endif
+
   if (ilevel == SOL_SOCKET)
     {
 #ifdef SO_LINGER
@@ -590,6 +607,12 @@ SCM_DEFINE (scm_setsockopt, "setsockopt", 4, 0, 0,
 	    "(ie.@: without @code{struct linger}), only @var{ENABLE} has an\n"
 	    "effect but the value in Guile is always a pair.\n"
 	    "@end defvar\n"
+            "\n"
+	    "@defvar SO_RCVTIMEO\n"
+	    "@defvarx SO_SNDTIMEO\n"
+	    "@var{value} is a pair of integers @code{(@var{SECONDS}\n"
+	    ". @var{MICROSECONDS})}.\n"
+	    "@end defvar\n"
 	    "\n"
 	    "@c  Note that we refer only to ``man ip'' here.  On GNU/Linux it's\n"
 	    "@c  ``man 7 ip'' but on NetBSD it's ``man 4 ip''.\n"
@@ -633,6 +656,8 @@ SCM_DEFINE (scm_setsockopt, "setsockopt", 4, 0, 0,
   struct ip_mreq opt_mreq;
 #endif
 
+  struct timeval opt_time;
+
   const void *optval = NULL;
   socklen_t optlen = 0;
 
@@ -682,6 +707,19 @@ SCM_DEFINE (scm_setsockopt, "setsockopt", 4, 0, 0,
     }
 #endif
 
+#if defined(SO_RCVTIMEO) && defined(SO_SNDTIMEO)
+  if (ioptname == SO_RCVTIMEO || ioptname == SO_SNDTIMEO)
+    {
+      SCM_ASSERT_TYPE (scm_is_pair (value), value, SCM_ARG4, FUNC_NAME,
+                       "value");
+      opt_time.tv_sec = scm_to_ulong (SCM_CAR (value));
+      opt_time.tv_usec = scm_to_ulong (SCM_CDR (value));
+
+      optlen = sizeof (opt_time);
+      optval = &opt_time;
+    }
+#endif
+
   if (optval == NULL)
     {
       /* Most options take an int.  */
@@ -813,10 +851,11 @@ scm_fill_sockaddr (int fam, SCM address, SCM *args, int which_arg,
 	struct sockaddr_un *soka;
 	int addr_size;
 	char *c_address;
+	size_t c_address_size;
 
 	scm_dynwind_begin (0);
 
-	c_address = scm_to_locale_string (address);
+	c_address = scm_to_locale_stringn (address, &c_address_size);
 	scm_dynwind_free (c_address);
 
 	/* the static buffer size in sockaddr_un seems to be arbitrary
@@ -826,12 +865,14 @@ scm_fill_sockaddr (int fam, SCM address, SCM *args, int which_arg,
 	   connect/bind etc., to fail.  sun_path is always the last
 	   member of the structure.  */
 	addr_size = sizeof (struct sockaddr_un)
-	  + MAX (0, strlen (c_address) + 1 - (sizeof soka->sun_path));
+	  + MAX (0, c_address_size + 1 - (sizeof soka->sun_path));
 	soka = (struct sockaddr_un *) scm_malloc (addr_size);
-	memset (soka, 0, addr_size);  /* for sun_len: see sin_len above. */
+	memset (soka, 0, addr_size);
 	soka->sun_family = AF_UNIX;
-	strcpy (soka->sun_path, c_address);
-	*size = SUN_LEN (soka);
+        /* we accept 0-bytes here (used for abstract sockets in Linux);
+           therefore do not use strlen() or SUN_LEN!  */
+	memcpy (soka->sun_path, c_address, c_address_size);
+	*size = offsetof (struct sockaddr_un, sun_path) + c_address_size;
 
 	scm_dynwind_end ();
 	return (struct sockaddr *) soka;
@@ -890,7 +931,7 @@ SCM_DEFINE (scm_connect, "connect", 2, 1, 1,
 
       free (soka);
       errno = save_errno;
-      if (errno == EINPROGRESS)
+      if (errno == EINPROGRESS || errno == EAGAIN)
         return SCM_BOOL_F;
       SCM_SYSERROR;
     }
@@ -1045,7 +1086,12 @@ _scm_from_sockaddr (const scm_t_max_sockaddr *address, unsigned addr_size,
 	if (addr_size <= offsetof (struct sockaddr_un, sun_path))
 	  SCM_SIMPLE_VECTOR_SET(result, 1, SCM_BOOL_F);
 	else
-	  SCM_SIMPLE_VECTOR_SET(result, 1, scm_from_locale_string (nad->sun_path));
+          {
+            size_t path_size = addr_size - offsetof (struct sockaddr_un, sun_path);
+            SCM_SIMPLE_VECTOR_SET (result, 1,
+                                   scm_from_locale_stringn (nad->sun_path,
+                                                            path_size));
+          }
       }
       break;
 #endif
@@ -1648,6 +1694,7 @@ scm_init_socket ()
   /* standard addresses.  */
 #ifdef INADDR_ANY
   scm_c_define ("INADDR_ANY", scm_from_ulong (INADDR_ANY));
+  scm_c_define ("IN6ADDR_ANY", scm_from_ulong (0));
 #endif
 #ifdef INADDR_BROADCAST
   scm_c_define ("INADDR_BROADCAST", scm_from_ulong (INADDR_BROADCAST));
@@ -1657,6 +1704,7 @@ scm_init_socket ()
 #endif
 #ifdef INADDR_LOOPBACK
   scm_c_define ("INADDR_LOOPBACK", scm_from_ulong (INADDR_LOOPBACK));
+  scm_c_define ("IN6ADDR_LOOPBACK", scm_from_ulong (1));
 #endif
 
   /* socket types.
@@ -1702,6 +1750,9 @@ scm_init_socket ()
 #ifdef IPPROTO_IP
   scm_c_define ("IPPROTO_IP", scm_from_int (IPPROTO_IP));
 #endif
+#ifdef IPPROTO_IPV6
+  scm_c_define ("IPPROTO_IPV6", scm_from_int (IPPROTO_IPV6));
+#endif
 #ifdef IPPROTO_TCP
   scm_c_define ("IPPROTO_TCP", scm_from_int (IPPROTO_TCP));
 #endif
@@ -1755,6 +1806,12 @@ scm_init_socket ()
 #ifdef SO_REUSEPORT				  /* new in Linux 3.9 */
   scm_c_define ("SO_REUSEPORT", scm_from_int (SO_REUSEPORT));
 #endif
+#ifdef SO_RCVTIMEO
+  scm_c_define ("SO_RCVTIMEO", scm_from_int (SO_RCVTIMEO));
+#endif
+#ifdef SO_SNDTIMEO
+  scm_c_define ("SO_SNDTIMEO", scm_from_int (SO_SNDTIMEO));
+#endif
 
   /* recv/send options.  */
 #ifdef MSG_DONTWAIT
@@ -1791,6 +1848,10 @@ scm_init_socket ()
   scm_c_define ("IP_MULTICAST_IF", scm_from_int ( IP_MULTICAST_IF));
 #endif
 
+#ifdef IPV6_V6ONLY
+  scm_c_define ("IPV6_V6ONLY", scm_from_int (IPV6_V6ONLY));
+#endif
+
   scm_add_feature ("socket");
 
 #include "socket.x"
