@@ -1,36 +1,40 @@
-/* Copyright (C) 1995,1996,1997,1998,2000,2001, 2006, 2008, 2011 Free Software Foundation, Inc.
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-1998,2000-2001,2006,2008,2011,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
+
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
 
-#include "libguile/_scm.h"
-#include "libguile/eq.h"
-#include "libguile/ports.h"
-#include "libguile/root.h"
-#include "libguile/smob.h"
-#include "libguile/deprecation.h"
+#include "boolean.h"
+#include "deprecation.h"
+#include "eq.h"
+#include "gsubr.h"
+#include "list.h"
+#include "ports.h"
+#include "smob.h"
+
+#include "variable.h"
+
 
-#include "libguile/validate.h"
-#include "libguile/variable.h"
 
 
 void
@@ -138,11 +142,5 @@ SCM_DEFINE (scm_variable_bound_p, "variable-bound?", 1, 0, 0,
 void
 scm_init_variable ()
 {
-#include "libguile/variable.x"
+#include "variable.x"
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
