@@ -1,6 +1,6 @@
 ;;; Continuation-passing style (CPS) intermediate language (IL)
 
-;; Copyright (C) 2013, 2014, 2015, 2017, 2018 Free Software Foundation, Inc.
+;; Copyright (C) 2013-2021 Free Software Foundation, Inc.
 
 ;;;; This library is free software; you can redistribute it and/or
 ;;;; modify it under the terms of the GNU Lesser General Public
@@ -89,15 +89,24 @@ references."
                          (add-uses args uses))
                         (($ $call proc args)
                          (add-use proc (add-uses args uses)))
+                        (($ $callk k proc args)
+                         (let ((uses (add-uses args uses)))
+                           (if proc
+                               (add-use proc uses)
+                               uses)))
+                        (($ $calli args callee)
+                         (add-uses args (add-use callee uses)))
                         (($ $primcall name param args)
                          (add-uses args uses))))
                      (($ $branch kf kt src op param args)
                       (add-uses args uses))
+                     (($ $switch kf kt* src arg)
+                      (add-use arg uses))
                      (($ $prompt k kh src escape? tag)
                       (add-use tag uses))
                      (($ $throw src op param args)
                       (add-uses args uses)))))
-                 (($ $kfun src meta self)
+                 (($ $kfun src meta (and self (not #f)))
                   (values (add-def self defs) uses))
                  (_ (values defs uses))))
              body empty-intset empty-intset))
