@@ -1,6 +1,6 @@
 ;;; srfi-43.scm -- SRFI 43 Vector library
 
-;;      Copyright (C) 2014 Free Software Foundation, Inc.
+;;      Copyright (C) 2014, 2018 Free Software Foundation, Inc.
 ;;
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -19,11 +19,12 @@
 ;;; Author: Mark H Weaver <mhw@netris.org>
 
 (define-module (srfi srfi-43)
+  #:use-module ((guile) #:select ((vector-copy . guile-vector-copy)))
   #:use-module (srfi srfi-1)
   #:use-module (srfi srfi-8)
   #:re-export (make-vector vector vector? vector-ref vector-set!
-                           vector-length)
-  #:replace (vector-copy vector-fill! list->vector vector->list)
+                           vector-length vector-fill! vector-copy!)
+  #:replace (vector-copy list->vector vector->list)
   #:export (vector-empty? vector= vector-unfold vector-unfold-right
                           vector-reverse-copy
                           vector-append vector-concatenate
@@ -35,16 +36,20 @@
                           vector-binary-search
                           vector-any vector-every
                           vector-swap! vector-reverse!
-                          vector-copy! vector-reverse-copy!
+                          vector-reverse-copy!
                           reverse-vector->list
                           reverse-list->vector))
 
 (cond-expand-provide (current-module) '(srfi-43))
 
-(define (error-from who msg . args)
-  (apply error
-         (string-append (symbol->string who) ": " msg)
-         args))
+(define-syntax error-from
+  (lambda (stx)
+    (syntax-case stx (quote)
+      ((_ 'who msg arg ...)
+       #`(error #,(string-append (symbol->string (syntax->datum #'who))
+                                 ": "
+                                 (syntax->datum #'msg))
+                arg ...)))))
 
 (define-syntax-rule (assert-nonneg-exact-integer k who)
   (unless (and (exact-integer? k)
@@ -198,9 +203,6 @@ error for the number of seeds to vary between iterations."
              (loop (- i 1) seeds))))
        v))))
 
-(define guile-vector-copy (@ (guile) vector-copy))
-
-;; TODO: Enhance Guile core 'vector-copy' to do this.
 (define vector-copy
   (case-lambda*
    "(vector-copy vec [start [end [fill]]]) -> vector
@@ -213,23 +215,20 @@ VEC, the slots in the new vector that obviously cannot be filled by
 elements from VEC are filled with FILL, whose default value is
 unspecified."
    ((v) (guile-vector-copy v))
-   ((v start)
-    (assert-vector v 'vector-copy)
-    (let ((len (vector-length v)))
-      (assert-valid-start start len 'vector-copy)
-      (let ((result (make-vector (- len start))))
-        (vector-move-left! v start len result 0)
-        result)))
+   ((v start) (guile-vector-copy v start))
    ((v start end #:optional (fill *unspecified*))
     (assert-vector v 'vector-copy)
     (let ((len (vector-length v)))
-      (unless (and (exact-integer? start)
-                   (exact-integer? end)
-                   (<= 0 start end))
-        (error-from 'vector-copy "invalid index range" start end))
-      (let ((result (make-vector (- end start) fill)))
-        (vector-move-left! v start (min end len) result 0)
-        result)))))
+      (if (<= end len)
+        (guile-vector-copy v start end)
+        (begin
+          (unless (and (exact-integer? start)
+                       (exact-integer? end)
+                       (<= 0 start end))
+            (error-from 'vector-copy "invalid index range" start end))
+          (let ((result (make-vector (- end start) fill)))
+            (vector-move-left! v start (min end len) result 0)
+            result)))))))
 
 (define vector-reverse-copy
   (let ()
@@ -546,7 +545,7 @@ left-to-right."
 
 (define vector-count
   (case-lambda
-    "(vector-count pred? vec1 vec2 ...) -> exact nonnegative integer
+    "(vector-count pred? vec1 vec2 ...) -> exact non-negative integer
 
 Count the number of indices i for which (PRED? VEC1[i] VEC2[i] ...)
 returns true, where i is less than the length of the shortest vector
@@ -586,7 +585,7 @@ passed."
 
 (define vector-index
   (case-lambda
-    "(vector-index pred? vec1 vec2 ...) -> exact nonnegative integer or #f
+    "(vector-index pred? vec1 vec2 ...) -> exact non-negative integer or #f
 
 Find and return the index of the first elements in VEC1 VEC2 ... that
 satisfy PRED?.  If no matching element is found by the end of the
@@ -624,7 +623,7 @@ shortest vector, return #f."
 
 (define vector-index-right
   (case-lambda
-    "(vector-index-right pred? vec1 vec2 ...) -> exact nonnegative integer or #f
+    "(vector-index-right pred? vec1 vec2 ...) -> exact non-negative integer or #f
 
 Find and return the index of the last elements in VEC1 VEC2 ... that
 satisfy PRED?, searching from right-to-left.  If no matching element
@@ -662,7 +661,7 @@ is found before the end of the shortest vector, return #f."
 
 (define vector-skip
   (case-lambda
-    "(vector-skip pred? vec1 vec2 ...) -> exact nonnegative integer or #f
+    "(vector-skip pred? vec1 vec2 ...) -> exact non-negative integer or #f
 
 Find and return the index of the first elements in VEC1 VEC2 ... that
 do not satisfy PRED?.  If no matching element is found by the end of
@@ -700,7 +699,7 @@ the shortest vector, return #f."
 
 (define vector-skip-right
   (case-lambda
-    "(vector-skip-right pred? vec1 vec2 ...) -> exact nonnegative integer or #f
+    "(vector-skip-right pred? vec1 vec2 ...) -> exact non-negative integer or #f
 
 Find and return the index of the last elements in VEC1 VEC2 ... that
 do not satisfy PRED?, searching from right-to-left.  If no matching
@@ -748,7 +747,7 @@ element is found before the end of the shortest vector, return #f."
                      ((positive? c) (loop lo i))
                      ((negative? c) (loop (+ i 1) hi)))))))
     (case-lambda
-      "(vector-binary-search vec value cmp [start [end]]) -> exact nonnegative integer or #f
+      "(vector-binary-search vec value cmp [start [end]]) -> exact non-negative integer or #f
 
 Find and return an index of VEC between START and END whose value is
 VALUE using a binary search.  If no matching element is found, return
@@ -868,34 +867,6 @@ Swap the values of the locations in VEC at I and J."
       (vector-set! vec i (vector-ref vec j))
       (vector-set! vec j tmp))))
 
-;; TODO: Enhance Guile core 'vector-fill!' to do this.
-(define vector-fill!
-  (let ()
-    (define guile-vector-fill!
-      (@ (guile) vector-fill!))
-    (define (%vector-fill! vec fill start end)
-      (let loop ((i start))
-        (when (< i end)
-          (vector-set! vec i fill)
-          (loop (+ i 1)))))
-    (case-lambda
-      "(vector-fill! vec fill [start [end]]) -> unspecified
-
-Assign the value of every location in VEC between START and END to
-FILL.  START defaults to 0 and END defaults to the length of VEC."
-      ((vec fill)
-       (guile-vector-fill! vec fill))
-      ((vec fill start)
-       (assert-vector vec 'vector-fill!)
-       (let ((len (vector-length vec)))
-         (assert-valid-start start len 'vector-fill!)
-         (%vector-fill! vec fill start len)))
-      ((vec fill start end)
-       (assert-vector vec 'vector-fill!)
-       (let ((len (vector-length vec)))
-         (assert-valid-range start end len 'vector-fill!)
-         (%vector-fill! vec fill start end))))))
-
 (define (%vector-reverse! vec start end)
   (let loop ((i start) (j (- end 1)))
     (when (< i j)
@@ -961,19 +932,6 @@ START defaults to 0 and END defaults to the length of VEC."
              (error-from 'copy! "would write past end of target"))
            (%copy! target tstart source sstart send)))))))
 
-(define-vector-copier! vector-copy!
-  "(vector-copy! target tstart source [sstart [send]]) -> unspecified
-
-Copy a block of elements from SOURCE to TARGET, both of which must be
-vectors, starting in TARGET at TSTART and starting in SOURCE at
-SSTART, ending when SEND - SSTART elements have been copied.  It is an
-error for TARGET to have a length less than TSTART + (SEND - SSTART).
-SSTART defaults to 0 and SEND defaults to the length of SOURCE."
-  (lambda (target tstart source sstart send)
-    (if (< tstart sstart)
-        (vector-move-left!  source sstart send target tstart)
-        (vector-move-right! source sstart send target tstart))))
-
 (define-vector-copier! vector-reverse-copy!
   "(vector-reverse-copy! target tstart source [sstart [send]]) -> unspecified
 
