@@ -1,20 +1,19 @@
 ;;; User interface messages
 
-;; Copyright (C) 2009, 2010, 2011, 2012 Free Software Foundation, Inc.
+;; Copyright (C) 2009-2012,2016,2018,2020-2021,2023 Free Software Foundation, Inc.
 
-;;; This library is free software; you can redistribute it and/or
-;;; modify it under the terms of the GNU Lesser General Public
-;;; License as published by the Free Software Foundation; either
-;;; version 3 of the License, or (at your option) any later version.
+;;; This library is free software; you can redistribute it and/or modify it
+;;; under the terms of the GNU Lesser General Public License as published by
+;;; the Free Software Foundation; either version 3 of the License, or (at
+;;; your option) any later version.
 ;;;
-;;; This library is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-;;; Lesser General Public License for more details.
+;;; This library is distributed in the hope that it will be useful, but
+;;; WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser
+;;; General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU Lesser General Public
-;;; License along with this library; if not, write to the Free Software
-;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 ;;;
@@ -42,12 +41,19 @@
 ;;;
 
 (define (location-string loc)
-  (if (pair? loc)
-      (format #f "~a:~a:~a"
-              (or (assoc-ref loc 'filename) "<stdin>")
-              (1+ (assoc-ref loc 'line))
-              (assoc-ref loc 'column))
-      "<unknown-location>"))
+  (define (format-loc file line column)
+    (format #f "~a:~a:~a"
+            (or file "<stdin>")
+            (1+ line)
+            column))
+  (match loc
+    (#(file line column)
+     (format-loc file line column))
+    ((? pair? loc)
+     (format-loc (assoc-ref loc 'filename)
+                 (assoc-ref loc 'line)
+                 (assoc-ref loc 'column)))
+    (_ "<unknown-location>")))
 
 
 ;;;
@@ -75,7 +81,7 @@
   (printer      warning-type-printer))
 
 (define %warning-types
-  ;; List of know warning types.
+  ;; List of known warning types.
   (map (lambda (args)
          (apply make-warning-type args))
 
@@ -109,12 +115,46 @@
                (emit port "~A: warning: possibly unused local top-level variable `~A'~%"
                      loc name)))
 
+           (unused-module
+            "report unused modules"
+            ,(lambda (port loc name definitely-unused?)
+               (if definitely-unused?
+                   (emit port "~A: warning: unused module ~a~%"
+                         loc name)
+                   (emit port "~A: warning: possibly unused module ~a~%"
+                         loc name))))
+
+           (shadowed-toplevel
+            "report shadowed top-level variables"
+            ,(lambda (port loc name previous-loc)
+               (emit port "~A: warning: shadows previous definition of `~A' at ~A~%"
+                     loc name
+                     (location-string previous-loc))))
+
            (unbound-variable
             "report possibly unbound variables"
             ,(lambda (port loc name)
                (emit port "~A: warning: possibly unbound variable `~A'~%"
                      loc name)))
 
+           (macro-use-before-definition
+            "report possibly mis-use of macros before they are defined"
+            ,(lambda (port loc name)
+               (emit port "~A: warning: macro `~A' used before definition~%"
+                     loc name)))
+
+           (use-before-definition
+            "report uses of top-levels before they are defined"
+            ,(lambda (port loc name)
+               (emit port "~A: warning: `~A' used before definition~%"
+                     loc name)))
+
+           (non-idempotent-definition
+            "report names that can refer to imports on first load, but module definitions on second load"
+            ,(lambda (port loc name)
+               (emit port "~A: warning: non-idempotent binding for `~A'.  When first loaded, value for `~A` comes from imported binding, but later module-local definition overrides it; any module reload would capture module-local binding rather than import.~%"
+                     loc name name)))
+
            (arity-mismatch
             "report procedure arity mismatches (wrong number of arguments)"
             ,(lambda (port loc name certain?)
@@ -126,6 +166,20 @@
                          "~A: warning: possibly wrong number of arguments to `~A'~%"
                          loc name))))
 
+           (duplicate-case-datum
+            "report a duplicate datum in a case expression"
+            ,(lambda (port loc datum clause case-expr)
+               (emit port
+                     "~A: warning: duplicate datum ~S in clause ~S of case expression ~S~%"
+                     loc datum clause case-expr)))
+
+           (bad-case-datum
+            "report a case datum that cannot be meaningfully compared using `eqv?'"
+            ,(lambda (port loc datum clause case-expr)
+               (emit port
+                     "~A: warning: datum ~S cannot be meaningfully compared using `eqv?' in clause ~S of case expression ~S~%"
+                     loc datum clause case-expr)))
+
            (format
             "report wrong number of arguments to `format'"
             ,(lambda (port loc . rest)
@@ -207,5 +261,3 @@ property alist) using the data in ARGS."
                args)
         (format port "~A: unknown warning type `~A': ~A~%"
                 (location-string location) type args))))
-
-;;; message.scm ends here
